﻿using HardwareShopContracts.BindingModels;
using HardwareShopContracts.ViewModels;
using HardwareShopDatabaseImplement.Models.ManyToMany;
using HardwareShopDatabaseImplement.Models.Worker;
using HardwareShopDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace HardwareShopDatabaseImplement.Models.Storekeeper
{
    public class Component : IComponentModel
    {
        public int Id { get; set; }

        [Required]
        public string ComponentName { get; set; } = string.Empty;

        [Required]
        public double Cost { get; set; }

        [Required]
        public int UserId { get; set; }

        [ForeignKey("ComponentId")]
        public virtual List<GoodComponent> Goods { get; set; } = new();

        [ForeignKey("ComponentId")]
        public virtual List<ComponentBuild> Builds { get; set; } = new();

        public virtual User User { get; set; } = null!;

        private Dictionary<int, (IBuildModel, int)>? _componentsBuilds = null;

        [NotMapped]
        public Dictionary<int, (IBuildModel, int)>? ComponentsBuilds
        {
            get
            {
                if (_componentsBuilds == null)
                {
                    _componentsBuilds = Builds
                            .ToDictionary(recCB => recCB.BuildId, recCB => (recCB.Build as IBuildModel, recCB.Count));
                }
                return _componentsBuilds;
            }
        }

        public static Component? Create(ComponentBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Component()
            {
                Id = model.Id,
                ComponentName = model.ComponentName,
                Cost = model.Cost,
                UserId = model.UserId
            };
        }

        public void Update(ComponentBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ComponentName = model.ComponentName;
            Cost = model.Cost;
        }

        public ComponentViewModel GetViewModel => new()
        {
            Id = Id,
            ComponentName = ComponentName,
            Cost = Cost,
            UserId = UserId,
            UserLogin = User.Login,
            ComponentsBuilds = ComponentsBuilds
        };

        public void UpdateBuilds(HardwareShopDatabase context, ComponentBindingModel model)
        {
            var componentBuilds = context.ComponentsBuilds.Where(rec => rec.ComponentId == model.Id).ToList();
            if (componentBuilds != null && componentBuilds.Count > 0)
            {   // удалили те, которых нет в модели
                context.ComponentsBuilds.RemoveRange(componentBuilds.Where(rec => !model.ComponentsBuilds.ContainsKey(rec.BuildId)));
                context.SaveChanges();
                // обновили количество у существующих записей
                foreach (var updateBuild in componentBuilds)
                {
                    updateBuild.Count = model.ComponentsBuilds[updateBuild.BuildId].Item2;
                    model.ComponentsBuilds.Remove(updateBuild.BuildId);
                }
                context.SaveChanges();
            }
            var component = context.Components.First(x => x.Id == Id);
            foreach (var cb in model.ComponentsBuilds)
            {
                context.ComponentsBuilds.Add(new ComponentBuild
                {
                    Component = component,
                    Build = context.Builds.First(x => x.Id == cb.Key),
                    Count = cb.Value.Item2
                });

                var build = context.Builds.First(x => x.Id == cb.Key);
                if (build != null)
                {
                    build.Price += cb.Value.Item2 * component.Cost;
                }
                context.SaveChanges();
            }
            _componentsBuilds = null;
        }
    }
}
